#include <stdint.h>
#include <built_in.h>
#include "resources.h"

/*
 * TFT pins
 ******************************************************************************/
char TFT_DataPort at GPIO_PORT_32_39;
sbit TFT_RST at GPIO_PIN42_bit;
sbit TFT_RS at GPIO_PIN40_bit;
sbit TFT_CS at GPIO_PIN41_bit;
sbit TFT_RD at GPIO_PIN46_bit;
sbit TFT_WR at GPIO_PIN47_bit;
sbit TFT_BLED at GPIO_PIN58_bit;

/*
 * Ozone CS pin
 ******************************************************************************/
sbit OZONE_CS at GPIO_PIN28_bit;

/*
 * Global vars
 ******************************************************************************/
const float    Rr    = 10000.0;
const float    Vref  = 5.0;
uint16_t       raw;
float          ozone;
char           text[ 20 ];

/*
 * Prototypes
 ******************************************************************************/
void Auxiliary_Code( void );
void Set_Index( unsigned short index );
void Write_Command( unsigned short cmd );
void Write_Data( unsigned int _data );
void system_init( void );
void display_init( void );
void display_result( void );
void measurement( void );
void calculation( void );

/*
 * Functions
 ******************************************************************************/
 void Auxiliary_Code() {
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Set_Index( unsigned short index ) {
  TFT_RS = 0;
  asm nop;
  TFT_DataPort = index;
  asm nop; asm nop;
  Auxiliary_Code();
}

void Write_Command( unsigned short cmd ) {
  TFT_RS = 1;
  asm nop;
  TFT_DataPort = cmd;
  asm nop; asm nop;
  Auxiliary_Code();
}

void Write_Data( unsigned int _data ) {
  TFT_RS = 1;
  asm nop;
  TFT_DataPort = Hi(_data);
  asm nop; asm nop;
  Auxiliary_Code();
  asm nop;
  TFT_DataPort = Lo(_data);
  asm nop; asm nop;
  Auxiliary_Code();
}

void system_init()
{
    GPIO_Digital_Output( &GPIO_PORT_24_31, _GPIO_PINMASK_4 );
    Delay_ms( 200 );

    SPIM1_Init_Advanced( _SPI_MASTER_CLK_RATIO_16, _SPI_CFG_PHASE_CAPTURE_RISING |
                         _SPI_CFG_POLARITY_IDLE_HIGH | _SPI_CFG_SS_AUTO_DISABLE |
                         _SPI_CFG_FIFO_DISABLE, _SPI_SS_LINE_NONE );
    Delay_ms( 200 );

    OZONE_CS = 1;
}

void display_init()
{
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Init_ILI9341_8bit( 320, 240 );
    TFT_BLED = 1;
    TFT_Set_Pen( CL_WHITE, 1 );
    TFT_Set_Brush( 1, CL_WHITE, 0, 0, 0, 0 );
    TFT_Set_Font( TFT_defaultFont, CL_BLACK, FO_HORIZONTAL );
    TFT_Fill_Screen( CL_WHITE );
    TFT_Set_Pen( CL_BLACK, 1 );
    TFT_Line(  20,  46, 300,  46 );
    TFT_Line(  20,  70, 300,  70 );
    TFT_Line(  20, 220, 300, 220 );
    TFT_Set_Pen( CL_WHITE, 1 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "Ozone click", 110, 14 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "Ozone ( PPB ) :", 100, 110 );
    TFT_Set_Font( &Tahoma15x16_Bold, CL_BLUE, FO_HORIZONTAL );
    TFT_Write_Text( "MiCS-2614", 125, 50 );
    TFT_Set_Font( &Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "EasyMx PRO v7 for STM32", 19, 223 );
    TFT_Set_Font( &Verdana12x13_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "www.mikroe.com", 200, 223 );
    TFT_Set_Font( &Tahoma15x16_Bold, CL_RED, FO_HORIZONTAL );
    delay_ms( 200 );
}

void display_result()
{
    FloatToStr( ozone, text );
    TFT_Rectangle( 100, 145, 300, 170 );
    TFT_Write_Text( text, 128, 145 );
}

void measurement()
{
    uint8_t msb;
    uint8_t lsb;

    OZONE_CS = 0;
    msb = SPIM_Rd_Ptr( 0xFF );
    lsb = SPIM_Rd_Ptr( 0xFF );
    OZONE_CS = 1;

    raw = ( msb & 0x1F );
    raw <<= 7;
    raw |= ( lsb >> 1 );
}
void calculation()
{
    float Vout;
    float Rs;
    float ratio;
    float lgPPM;

    Vout    = ( float )raw / 4096.0 * Vref;
    FloatToStr( Vout, text );
    Rs = Rr / ( Vref - Vout ) * Vout;
    FloatToStr( Rs, text );
    ratio   = Rs / Rr;
    ozone   = ( ( log10( ratio ) *  -0.8 ) + 0.9 ) * 1000;  // Parts per billion
}

/*
 * Main
 ******************************************************************************/
void main()
{
    system_init();
    display_init();

    while( 1 )
    {
        measurement();
        calculation();
        display_result();
        delay_ms( 500 );
    }
}